(function() {
  var Declaration, utils;

  utils = require('./utils');

  Declaration = (function() {
    function Declaration(rule, number, node) {
      var separator;
      this.rule = rule;
      this.number = number;
      this.node = node;
      this.prop = this.node.property;
      this.value = this.node.value;
      if (this.prop[0] === '-') {
        separator = this.prop.indexOf('-', 1) + 1;
        this.prefix = this.prop.slice(0, separator);
        this.unprefixed = this.prop.slice(separator);
      } else {
        this.unprefixed = this.prop;
      }
      this.valuesCache = {};
    }

    Declaration.prototype.valueContain = function(strings) {
      var _this = this;
      return strings.some(function(i) {
        return _this.value.indexOf(i) !== -1;
      });
    };

    Declaration.prototype.prefixProp = function(prefix) {
      if (this.rule.contain(prefix + this.unprefixed)) {
        return;
      }
      return this.insertBefore(prefix + this.unprefixed, this.value);
    };

    Declaration.prototype.insertBefore = function(prop, value) {
      var clone;
      if (this.rule.contain(prop, value)) {
        return;
      }
      clone = utils.clone(this.node, {
        property: prop,
        value: value
      });
      this.rule.add(this.number, clone);
      return this.number += 1;
    };

    Declaration.prototype.remove = function() {
      return this.rule.remove(this.number);
    };

    Declaration.prototype.prefixValue = function(prefix, value) {
      var val;
      val = this.valuesCache[prefix] || this.value;
      return this.valuesCache[prefix] = value.addPrefix(prefix, val);
    };

    Declaration.prototype.setValue = function(value) {
      return this.value = this.node.value = value;
    };

    Declaration.prototype.saveValues = function() {
      var prefix, prefixed, value, _ref, _results;
      _ref = this.valuesCache;
      _results = [];
      for (prefix in _ref) {
        value = _ref[prefix];
        if (this.rule.prefix && prefix !== this.rule.prefix) {
          continue;
        }
        if (prefixed = this.rule.byProp(prefix + this.unprefixed)) {
          _results.push(prefixed.setValue(value));
        } else {
          _results.push(this.insertBefore(this.prop, value));
        }
      }
      return _results;
    };

    return Declaration;

  })();

  module.exports = Declaration;

}).call(this);
